#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# System modules
from astropy.modeling import models, fitting
import numpy as np
from astropy.time import Time

# Custom modules
from utils.logger import log

surface = models.Const2D(amplitude=950)
moffat2d = models.Moffat2D(amplitude=1000)
gaussian2d = models.Gaussian2D(amplitude=1000)
fit_g = fitting.LevMarLSQFitter()


def fit_box2d(img, model="moffat"):
    """Fit an image with a 2D model. Default moffat, else gaussian.
    """

    y, x = np.mgrid[-img.shape[0]//2+1:img.shape[0]//2+1,
                    -img.shape[1]//2+1:img.shape[1]//2+1]

    if model == "moffat":
        f_init = surface + moffat2d
    else:
        f_init = surface + gaussian2d

    #log.debug(f"{x.shape}, {y.shape}, {img.shape}")

    xy_fit = fit_g(f_init, x, y, img)

    if model == "moffat":
        fwhm = 2*np.abs(xy_fit.gamma_1)*np.sqrt(2**(1/xy_fit.alpha_1)-1)
    else:
        fwhm = 2*np.sqrt(2*np.log(2))*(xy_fit.x_stddev_1)

    xy_fit.fwhm = fwhm

    log.debug(xy_fit)

    return xy_fit


def skyflat_exptime(average_counts_per_exptime, expected_counts=50000,
                    binning=1, repeat=20, min_exp=0.1, max_exp=60):
    """Calculate the exptime of sky flat images to obtain a given number
    of expected counts. Based on Tyson&Gal
    """

    now = Time.now()

    # k: relative time rate of change of sky surface brightness (sec^-1)
    # hour < 12: 'twilight', hour > 12: 'dawn'
    hour = now.datetime.hour
    k = 0.091/60

    if hour > 12:
        k = -k

    # tau: duration of astronomical twilight normalized to spring equinox at lat = +-30
    season = now.datetime.month

    if season in [12, 1, 2]: #  'winter'
        tau = 1.33
    elif season in [6, 7, 8]: # 'summer'
        tau = 1.91
    else: # 'sping' or 'autumn'
        tau = 1.23

    readout_time = 14.0/(binning**2) # In sec

    # Let's say we took an image with a given average counts / exptime
    t1 = expected_counts / average_counts_per_exptime  # calcolo primo tempo / prima esposizione

    a = 10**(k/tau)
    S = (a**(t1)-1)/np.log(a)

    # Requested flats
    exptimes = []
    for i in range(repeat):

        t2 = np.log(a**(t1+readout_time)+S*np.log(a))/np.log(a)
        exptime = t2 - (t1 + readout_time) # exptime so that counts / pixel is constant (in sec)

        if exptime < min_exp or exptime > max_exp:
            break
        else:
            exptimes.append(round(exptime,2))
            t1 = t2 # Iterative proess
    
    return exptimes
